@@ -375,9 +375,7 @@ public final class FuryBuilder {
       }
     }
     if (language != Language.JAVA) {
-      stringRefIgnored = true;
-      longEncoding = LongEncoding.PVL;
-      compressInt = true;
+      stringRefIgnored = false;
     }
     if (ENABLE_CLASS_REGISTRATION_FORCIBLY) {
       if (!requireClassRegistration) {
@@ -424,9 +422,6 @@ public final class FuryBuilder {
       if (metaShareEnabled == null) {
         metaShareEnabled = false;
       }
-      if (language != Language.JAVA) {
-        checkClassVersion = true;
-      }
     }
     if (!requireClassRegistration) {
       LOG.warn(
