@@ -30,6 +30,7 @@ import static org.apache.fury.util.MathUtils.toInt;
 
 import java.lang.reflect.Field;
 import java.util.ArrayList;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
 import java.util.List;
@@ -43,6 +44,7 @@ import org.apache.fury.meta.ClassDef.FieldInfo;
 import org.apache.fury.meta.ClassDef.FieldType;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.resolver.ClassResolver;
+import org.apache.fury.type.Descriptor;
 import org.apache.fury.type.DescriptorGrouper;
 import org.apache.fury.util.MurmurHash3;
 
@@ -53,12 +55,16 @@ import org.apache.fury.util.MurmurHash3;
  */
 class ClassDefEncoder {
   static List<Field> buildFields(Fury fury, Class<?> cls, boolean resolveParent) {
+    Comparator<Descriptor> comparator =
+        DescriptorGrouper.getPrimitiveComparator(fury.compressInt(), fury.compressLong());
     DescriptorGrouper descriptorGrouper =
-        fury.getClassResolver()
-            .createDescriptorGrouper(
-                fury.getClassResolver().getAllDescriptorsMap(cls, resolveParent).values(),
-                false,
-                Function.identity());
+        new DescriptorGrouper(
+            fury.getClassResolver()::isMonomorphic,
+            fury.getClassResolver().getAllDescriptorsMap(cls, resolveParent).values(),
+            false,
+            Function.identity(),
+            comparator,
+            DescriptorGrouper.COMPARATOR_BY_TYPE_AND_NAME);
     List<Field> fields = new ArrayList<>();
     descriptorGrouper
         .getPrimitiveDescriptors()
@@ -131,29 +137,20 @@ class ClassDefEncoder {
     MemoryBuffer classDefBuf = MemoryBuffer.newHeapBuffer(128);
     for (Map.Entry<String, List<FieldInfo>> entry : classLayers.entrySet()) {
       String className = entry.getKey();
+      Class<?> currentType = getType(type, className);
       List<FieldInfo> fields = entry.getValue();
       // | num fields + register flag | header + package name | header + class name
       // | header + type id + field name | next field info | ... |
       int currentClassHeader = (fields.size() << 1);
-      if (classResolver.isRegisteredById(type)) {
+      if (classResolver.isRegistered(currentType)) {
         currentClassHeader |= 1;
         classDefBuf.writeVarUint32Small7(currentClassHeader);
-        classDefBuf.writeVarUint32Small7(classResolver.getRegisteredClassId(type));
+        classDefBuf.writeVarUint32Small7(classResolver.getRegisteredClassId(currentType));
       } else {
         classDefBuf.writeVarUint32Small7(currentClassHeader);
-        Class<?> currentType = getType(type, className);
-        String ns, typename;
-        if (classResolver.isRegisteredByName(type)) {
-          Tuple2<String, String> nameTuple = classResolver.getRegisteredNameTuple(type);
-          ns = nameTuple.f0;
-          typename = nameTuple.f1;
-        } else {
-          Tuple2<String, String> encoded = Encoders.encodePkgAndClass(currentType);
-          ns = encoded.f0;
-          typename = encoded.f1;
-        }
-        writePkgName(classDefBuf, ns);
-        writeTypeName(classDefBuf, typename);
+        Tuple2<String, String> encoded = Encoders.encodePkgAndClass(currentType);
+        writePkgName(classDefBuf, encoded.f0);
+        writeTypeName(classDefBuf, encoded.f1);
       }
       writeFieldsInfo(classDefBuf, fields);
     }
