@@ -20,8 +20,7 @@
 package org.apache.fury.resolver;
 
 import static org.apache.fury.meta.Encoders.GENERIC_ENCODER;
-import static org.apache.fury.meta.Encoders.PACKAGE_DECODER;
-import static org.apache.fury.meta.Encoders.TYPE_NAME_DECODER;
+import static org.apache.fury.meta.Encoders.PACKAGE_ENCODER;
 
 import org.apache.fury.collection.Tuple2;
 import org.apache.fury.config.Language;
@@ -39,11 +38,11 @@ import org.apache.fury.util.function.Functions;
  */
 public class ClassInfo {
   final Class<?> cls;
-  final MetaStringBytes fullNameBytes;
-  final MetaStringBytes namespaceBytes;
-  final MetaStringBytes typeNameBytes;
+  final MetaStringBytes fullClassNameBytes;
+  final MetaStringBytes packageNameBytes;
+  final MetaStringBytes classNameBytes;
   final boolean isDynamicGeneratedClass;
-  int xtypeId;
+  final MetaStringBytes typeTagBytes;
   Serializer<?> serializer;
   // use primitive to avoid boxing
   // class id must be less than Integer.MAX_VALUE/2 since we use bit 0 as class id flag.
@@ -53,42 +52,42 @@ public class ClassInfo {
 
   ClassInfo(
       Class<?> cls,
-      MetaStringBytes fullNameBytes,
-      MetaStringBytes namespaceBytes,
-      MetaStringBytes typeNameBytes,
+      MetaStringBytes fullClassNameBytes,
+      MetaStringBytes packageNameBytes,
+      MetaStringBytes classNameBytes,
       boolean isDynamicGeneratedClass,
+      MetaStringBytes typeTagBytes,
       Serializer<?> serializer,
-      short classId,
-      short xtypeId) {
+      short classId) {
     this.cls = cls;
-    this.fullNameBytes = fullNameBytes;
-    this.namespaceBytes = namespaceBytes;
-    this.typeNameBytes = typeNameBytes;
+    this.fullClassNameBytes = fullClassNameBytes;
+    this.packageNameBytes = packageNameBytes;
+    this.classNameBytes = classNameBytes;
     this.isDynamicGeneratedClass = isDynamicGeneratedClass;
-    this.xtypeId = xtypeId;
+    this.typeTagBytes = typeTagBytes;
     this.serializer = serializer;
     this.classId = classId;
     if (cls != null && classId == ClassResolver.NO_CLASS_ID) {
-      Preconditions.checkArgument(typeNameBytes != null);
+      Preconditions.checkArgument(classNameBytes != null);
     }
   }
 
   ClassInfo(
       ClassResolver classResolver,
       Class<?> cls,
+      String tag,
       Serializer<?> serializer,
-      short classId,
-      short xtypeId) {
+      short classId) {
     this.cls = cls;
     this.serializer = serializer;
     needToWriteClassDef = serializer != null && classResolver.needToWriteClassDef(serializer);
     MetaStringResolver metaStringResolver = classResolver.getMetaStringResolver();
     if (cls != null && classResolver.getFury().getLanguage() != Language.JAVA) {
-      this.fullNameBytes =
+      this.fullClassNameBytes =
           metaStringResolver.getOrCreateMetaStringBytes(
               GENERIC_ENCODER.encode(cls.getName(), Encoding.UTF_8));
     } else {
-      this.fullNameBytes = null;
+      this.fullClassNameBytes = null;
     }
     // When `classId == ClassResolver.REPLACE_STUB_ID` was established,
     // means only classes are serialized, not the instance. If we
@@ -97,15 +96,21 @@ public class ClassInfo {
         && (classId == ClassResolver.NO_CLASS_ID || classId == ClassResolver.REPLACE_STUB_ID)) {
       // REPLACE_STUB_ID for write replace class in `ClassSerializer`.
       Tuple2<String, String> tuple2 = Encoders.encodePkgAndClass(cls);
-      this.namespaceBytes =
+      this.packageNameBytes =
           metaStringResolver.getOrCreateMetaStringBytes(Encoders.encodePackage(tuple2.f0));
-      this.typeNameBytes =
+      this.classNameBytes =
           metaStringResolver.getOrCreateMetaStringBytes(Encoders.encodeTypeName(tuple2.f1));
     } else {
-      this.namespaceBytes = null;
-      this.typeNameBytes = null;
+      this.packageNameBytes = null;
+      this.classNameBytes = null;
+    }
+    if (tag != null) {
+      this.typeTagBytes =
+          metaStringResolver.getOrCreateMetaStringBytes(
+              PACKAGE_ENCODER.encode(tag, Encoding.UTF_8));
+    } else {
+      this.typeTagBytes = null;
     }
-    this.xtypeId = xtypeId;
     this.classId = classId;
     if (cls != null) {
       boolean isLambda = Functions.isLambda(cls);
@@ -130,8 +135,12 @@ public class ClassInfo {
     return classId;
   }
 
-  public int getXtypeId() {
-    return xtypeId;
+  public MetaStringBytes getPackageNameBytes() {
+    return packageNameBytes;
+  }
+
+  public MetaStringBytes getClassNameBytes() {
+    return classNameBytes;
   }
 
   @SuppressWarnings("unchecked")
@@ -139,30 +148,18 @@ public class ClassInfo {
     return (Serializer<T>) serializer;
   }
 
-  public void setSerializer(Serializer<?> serializer) {
-    this.serializer = serializer;
-  }
-
   void setSerializer(ClassResolver resolver, Serializer<?> serializer) {
     this.serializer = serializer;
     needToWriteClassDef = serializer != null && resolver.needToWriteClassDef(serializer);
   }
 
-  public String decodeNamespace() {
-    return namespaceBytes.decode(PACKAGE_DECODER);
-  }
-
-  public String decodeTypeName() {
-    return typeNameBytes.decode(TYPE_NAME_DECODER);
-  }
-
   @Override
   public String toString() {
     return "ClassInfo{"
         + "cls="
         + cls
         + ", fullClassNameBytes="
-        + fullNameBytes
+        + fullClassNameBytes
         + ", isDynamicGeneratedClass="
         + isDynamicGeneratedClass
         + ", serializer="
