@@ -82,7 +82,6 @@ public final class MetaStringResolver {
       }
       dynamicWrittenMetaString[id] = byteString;
       int length = byteString.bytes.length;
-      // last bit `1` indicates class is written by name instead of registered id.
       buffer.writeVarUint32Small7(length << 2 | 0b1);
       if (length > SMALL_STRING_THRESHOLD) {
         buffer.writeInt64(byteString.hashCode);
@@ -91,7 +90,6 @@ public final class MetaStringResolver {
       }
       buffer.writeBytes(byteString.bytes);
     } else {
-      // last bit `1` indicates class is written by name instead of registered id.
       buffer.writeVarUint32Small7(((id + 1) << 2) | 0b11);
     }
   }
@@ -222,8 +220,12 @@ public final class MetaStringResolver {
   }
 
   private MetaStringBytes readSmallMetaStringBytes(MemoryBuffer buffer, int len) {
-    long v1, v2 = 0;
     byte encoding = buffer.readByte();
+    if (len == 0) {
+      assert encoding == MetaString.Encoding.UTF_8.getValue();
+      return MetaStringBytes.EMPTY;
+    }
+    long v1, v2 = 0;
     if (len <= 8) {
       v1 = buffer.readBytesAsInt64(len);
     } else {
@@ -239,8 +241,12 @@ public final class MetaStringResolver {
 
   private MetaStringBytes readSmallMetaStringBytes(
       MemoryBuffer buffer, MetaStringBytes cache, int len) {
-    long v1, v2 = 0;
     byte encoding = buffer.readByte();
+    if (len == 0) {
+      assert encoding == MetaString.Encoding.UTF_8.getValue();
+      return MetaStringBytes.EMPTY;
+    }
+    long v1, v2 = 0;
     if (len <= 8) {
       v1 = buffer.readBytesAsInt64(len);
     } else {
@@ -262,8 +268,7 @@ public final class MetaStringResolver {
     LittleEndian.putInt64(data, 0, v1);
     LittleEndian.putInt64(data, 8, v2);
     long hashCode = MurmurHash3.murmurhash3_x64_128(data, 0, len, 47)[0];
-    hashCode = Math.abs(hashCode);
-    hashCode = (hashCode & 0xffffffffffffff00L) | encoding;
+    hashCode = ((hashCode) & 0xffffffffffffff00L) | encoding;
     MetaStringBytes metaStringBytes = new MetaStringBytes(Arrays.copyOf(data, len), hashCode);
     longLongMap.put(v1, v2, metaStringBytes);
     return metaStringBytes;
