@@ -48,6 +48,7 @@ import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.memory.Platform;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.resolver.ClassResolver;
+import org.apache.fury.type.Type;
 import org.apache.fury.util.ExceptionUtils;
 import org.apache.fury.util.GraalvmSupport;
 import org.apache.fury.util.StringUtils;
@@ -177,18 +178,28 @@ public class Serializers {
   }
 
   public abstract static class CrossLanguageCompatibleSerializer<T> extends Serializer<T> {
+    private final short typeId;
 
-    public CrossLanguageCompatibleSerializer(Fury fury, Class<T> cls) {
+    public CrossLanguageCompatibleSerializer(Fury fury, Class<T> cls, short typeId) {
       super(fury, cls);
+      this.typeId = typeId;
     }
 
     public CrossLanguageCompatibleSerializer(
-        Fury fury, Class<T> cls, boolean needToWriteRef, boolean immutable) {
-      super(fury, cls, needToWriteRef, immutable);
+        Fury fury, Class<T> cls, short typeId, boolean needToWriteRef) {
+      super(fury, cls, needToWriteRef);
+      this.typeId = typeId;
     }
 
-    public CrossLanguageCompatibleSerializer(Fury fury, Class<T> cls, boolean needToWriteRef) {
-      super(fury, cls, needToWriteRef);
+    public CrossLanguageCompatibleSerializer(
+        Fury fury, Class<T> cls, short typeId, boolean needToWriteRef, boolean immutable) {
+      super(fury, cls, needToWriteRef, immutable);
+      this.typeId = typeId;
+    }
+
+    @Override
+    public short getXtypeId() {
+      return typeId;
     }
 
     @Override
@@ -228,7 +239,12 @@ public class Serializers {
 
     @Override
     public void xwrite(MemoryBuffer buffer, T value) {
-      stringSerializer.writeString(buffer, value.toString());
+      stringSerializer.writeUTF8String(buffer, value.toString());
+    }
+
+    @Override
+    public short getXtypeId() {
+      return (short) -Type.STRING.getId();
     }
 
     @Override
@@ -276,7 +292,7 @@ public class Serializers {
 
     @Override
     public StringBuilder xread(MemoryBuffer buffer) {
-      return new StringBuilder(stringSerializer.readString(buffer));
+      return new StringBuilder(stringSerializer.readUTF8String(buffer));
     }
   }
 
@@ -299,7 +315,7 @@ public class Serializers {
 
     @Override
     public StringBuffer xread(MemoryBuffer buffer) {
-      return new StringBuffer(stringSerializer.readString(buffer));
+      return new StringBuffer(stringSerializer.readUTF8String(buffer));
     }
   }
 
@@ -561,20 +577,19 @@ public class Serializers {
   }
 
   public static void registerDefaultSerializers(Fury fury) {
-    ClassResolver resolver = fury.getClassResolver();
-    resolver.registerSerializer(Class.class, new ClassSerializer(fury));
-    resolver.registerSerializer(StringBuilder.class, new StringBuilderSerializer(fury));
-    resolver.registerSerializer(StringBuffer.class, new StringBufferSerializer(fury));
-    resolver.registerSerializer(BigInteger.class, new BigIntegerSerializer(fury));
-    resolver.registerSerializer(BigDecimal.class, new BigDecimalSerializer(fury));
-    resolver.registerSerializer(AtomicBoolean.class, new AtomicBooleanSerializer(fury));
-    resolver.registerSerializer(AtomicInteger.class, new AtomicIntegerSerializer(fury));
-    resolver.registerSerializer(AtomicLong.class, new AtomicLongSerializer(fury));
-    resolver.registerSerializer(AtomicReference.class, new AtomicReferenceSerializer(fury));
-    resolver.registerSerializer(Currency.class, new CurrencySerializer(fury));
-    resolver.registerSerializer(URI.class, new URISerializer(fury));
-    resolver.registerSerializer(Pattern.class, new RegexSerializer(fury));
-    resolver.registerSerializer(UUID.class, new UUIDSerializer(fury));
-    resolver.registerSerializer(Object.class, new EmptyObjectSerializer(fury));
+    fury.registerSerializer(Class.class, new ClassSerializer(fury));
+    fury.registerSerializer(StringBuilder.class, new StringBuilderSerializer(fury));
+    fury.registerSerializer(StringBuffer.class, new StringBufferSerializer(fury));
+    fury.registerSerializer(BigInteger.class, new BigIntegerSerializer(fury));
+    fury.registerSerializer(BigDecimal.class, new BigDecimalSerializer(fury));
+    fury.registerSerializer(AtomicBoolean.class, new AtomicBooleanSerializer(fury));
+    fury.registerSerializer(AtomicInteger.class, new AtomicIntegerSerializer(fury));
+    fury.registerSerializer(AtomicLong.class, new AtomicLongSerializer(fury));
+    fury.registerSerializer(AtomicReference.class, new AtomicReferenceSerializer(fury));
+    fury.registerSerializer(Currency.class, new CurrencySerializer(fury));
+    fury.registerSerializer(URI.class, new URISerializer(fury));
+    fury.registerSerializer(Pattern.class, new RegexSerializer(fury));
+    fury.registerSerializer(UUID.class, new UUIDSerializer(fury));
+    fury.registerSerializer(Object.class, new EmptyObjectSerializer(fury));
   }
 }
