@@ -42,6 +42,7 @@ import org.apache.fury.memory.LittleEndian;
 import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.memory.Platform;
 import org.apache.fury.reflect.ReflectionUtils;
+import org.apache.fury.type.Type;
 import org.apache.fury.util.MathUtils;
 import org.apache.fury.util.Preconditions;
 import org.apache.fury.util.StringEncodingUtils;
@@ -120,6 +121,11 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     writeNumUtf16BytesForUtf8Encoding = fury.getConfig().writeNumUtf16BytesForUtf8Encoding();
   }
 
+  @Override
+  public short getXtypeId() {
+    return Type.STRING.getId();
+  }
+
   @Override
   public void write(MemoryBuffer buffer, String value) {
     writeJavaString(buffer, value);
@@ -127,7 +133,7 @@ public final class StringSerializer extends ImmutableSerializer<String> {
 
   @Override
   public void xwrite(MemoryBuffer buffer, String value) {
-    writeJavaString(buffer, value);
+    writeUTF8String(buffer, value);
   }
 
   @Override
@@ -137,52 +143,68 @@ public final class StringSerializer extends ImmutableSerializer<String> {
 
   @Override
   public String xread(MemoryBuffer buffer) {
-    return readJavaString(buffer);
+    return readUTF8String(buffer);
   }
 
   public void writeString(MemoryBuffer buffer, String value) {
-    writeJavaString(buffer, value);
+    if (isJava) {
+      writeJavaString(buffer, value);
+    } else {
+      writeUTF8String(buffer, value);
+    }
   }
 
   public Expression writeStringExpr(Expression strSerializer, Expression buffer, Expression str) {
-    if (STRING_VALUE_FIELD_IS_BYTES) {
-      if (compressString) {
-        return new Invoke(strSerializer, "writeCompressedBytesString", buffer, str);
+    if (isJava) {
+      if (STRING_VALUE_FIELD_IS_BYTES) {
+        if (compressString) {
+          return new Invoke(strSerializer, "writeCompressedBytesString", buffer, str);
+        } else {
+          return new StaticInvoke(StringSerializer.class, "writeBytesString", buffer, str);
+        }
       } else {
-        return new StaticInvoke(StringSerializer.class, "writeBytesString", buffer, str);
+        if (!STRING_VALUE_FIELD_IS_CHARS) {
+          throw new UnsupportedOperationException();
+        }
+        if (compressString) {
+          return new Invoke(strSerializer, "writeCompressedCharsString", buffer, str);
+        } else {
+          return new Invoke(strSerializer, "writeCharsString", buffer, str);
+        }
       }
     } else {
-      if (!STRING_VALUE_FIELD_IS_CHARS) {
-        throw new UnsupportedOperationException();
-      }
-      if (compressString) {
-        return new Invoke(strSerializer, "writeCompressedCharsString", buffer, str);
-      } else {
-        return new Invoke(strSerializer, "writeCharsString", buffer, str);
-      }
+      return new Invoke(strSerializer, "writeUTF8String", buffer, str);
     }
   }
 
   public String readString(MemoryBuffer buffer) {
-    return readJavaString(buffer);
+    if (isJava) {
+      return readJavaString(buffer);
+    } else {
+      return readUTF8String(buffer);
+    }
   }
 
   public Expression readStringExpr(Expression strSerializer, Expression buffer) {
-    if (STRING_VALUE_FIELD_IS_BYTES) {
-      if (compressString) {
-        return new Invoke(strSerializer, "readCompressedBytesString", STRING_TYPE, buffer);
+    if (isJava) {
+      if (STRING_VALUE_FIELD_IS_BYTES) {
+        if (compressString) {
+          return new Invoke(strSerializer, "readCompressedBytesString", STRING_TYPE, buffer);
+        } else {
+          return new Invoke(strSerializer, "readBytesString", STRING_TYPE, buffer);
+        }
       } else {
-        return new Invoke(strSerializer, "readBytesString", STRING_TYPE, buffer);
+        if (!STRING_VALUE_FIELD_IS_CHARS) {
+          throw new UnsupportedOperationException();
+        }
+        if (compressString) {
+          return new Invoke(strSerializer, "readCompressedCharsString", STRING_TYPE, buffer);
+        } else {
+          return new Invoke(strSerializer, "readCharsString", STRING_TYPE, buffer);
+        }
       }
     } else {
-      if (!STRING_VALUE_FIELD_IS_CHARS) {
-        throw new UnsupportedOperationException();
-      }
-      if (compressString) {
-        return new Invoke(strSerializer, "readCompressedCharsString", STRING_TYPE, buffer);
-      } else {
-        return new Invoke(strSerializer, "readCharsString", STRING_TYPE, buffer);
-      }
+      return new Invoke(strSerializer, "readUTF8String", STRING_TYPE, buffer);
     }
   }
 
@@ -273,6 +295,13 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     }
   }
 
+  @CodegenInvoke
+  public void writeUTF8String(MemoryBuffer buffer, String value) {
+    byte[] bytes = value.getBytes(StandardCharsets.UTF_8);
+    buffer.writeVarUint32(bytes.length);
+    buffer.writeBytes(bytes);
+  }
+
   // Invoked by fury JIT
   public String readJavaString(MemoryBuffer buffer) {
     if (STRING_VALUE_FIELD_IS_BYTES) {
@@ -366,6 +395,24 @@ public final class StringSerializer extends ImmutableSerializer<String> {
     }
   }
 
+  @CodegenInvoke
+  public String readUTF8String(MemoryBuffer buffer) {
+    int numBytes = buffer.readVarUint32Small14();
+    buffer.checkReadableBytes(numBytes);
+    final byte[] targetArray = buffer.getHeapMemory();
+    if (targetArray != null) {
+      String str =
+          new String(
+              targetArray, buffer._unsafeHeapReaderIndex(), numBytes, StandardCharsets.UTF_8);
+      buffer.increaseReaderIndex(numBytes);
+      return str;
+    } else {
+      final byte[] tmpArray = getByteArray(numBytes);
+      buffer.readBytes(tmpArray, 0, numBytes);
+      return new String(tmpArray, 0, numBytes, StandardCharsets.UTF_8);
+    }
+  }
+
   public char[] readCharsLatin1(MemoryBuffer buffer, int numBytes) {
     buffer.checkReadableBytes(numBytes);
     byte[] srcArray = buffer.getHeapMemory();
