@@ -41,7 +41,7 @@ import java.util.GregorianCalendar;
 import java.util.TimeZone;
 import org.apache.fury.Fury;
 import org.apache.fury.memory.MemoryBuffer;
-import org.apache.fury.resolver.ClassResolver;
+import org.apache.fury.type.Type;
 import org.apache.fury.util.DateTimeUtils;
 
 /** Serializers for all time related types. */
@@ -87,16 +87,6 @@ public class TimeSerializers {
       return newInstance(buffer.readInt64());
     }
 
-    @Override
-    public void xwrite(MemoryBuffer buffer, T value) {
-      buffer.writeInt64(value.getTime());
-    }
-
-    @Override
-    public T xread(MemoryBuffer buffer) {
-      return newInstance(buffer.readInt64());
-    }
-
     protected abstract T newInstance(long time);
   }
 
@@ -162,14 +152,17 @@ public class TimeSerializers {
   }
 
   public static final class TimestampSerializer extends TimeSerializer<Timestamp> {
+    private final short typeId;
 
     public TimestampSerializer(Fury fury) {
       // conflict with instant
       super(fury, Timestamp.class);
+      typeId = (short) -Type.TIMESTAMP.getId();
     }
 
     public TimestampSerializer(Fury fury, boolean needToWriteRef) {
       super(fury, Timestamp.class, needToWriteRef);
+      typeId = (short) -Type.TIMESTAMP.getId();
     }
 
     @Override
@@ -182,6 +175,11 @@ public class TimeSerializers {
       return DateTimeUtils.toJavaTimestamp(buffer.readInt64());
     }
 
+    @Override
+    public short getXtypeId() {
+      return typeId;
+    }
+
     @Override
     public void write(MemoryBuffer buffer, Timestamp value) {
       long time = value.getTime() - (value.getNanos() / 1_000_000);
@@ -211,6 +209,11 @@ public class TimeSerializers {
       super(fury, LocalDate.class, needToWriteRef);
     }
 
+    @Override
+    public short getXtypeId() {
+      return Type.DATE32.getId();
+    }
+
     @Override
     public void xwrite(MemoryBuffer buffer, LocalDate value) {
       // TODO use java encoding to support larger range.
@@ -255,6 +258,11 @@ public class TimeSerializers {
       super(fury, Instant.class, needToWriteRef);
     }
 
+    @Override
+    public short getXtypeId() {
+      return Type.TIMESTAMP.getId();
+    }
+
     @Override
     public void xwrite(MemoryBuffer buffer, Instant value) {
       // FIXME JDK17 may have higher precision than millisecond
@@ -681,23 +689,22 @@ public class TimeSerializers {
   }
 
   public static void registerDefaultSerializers(Fury fury) {
-    ClassResolver resolver = fury.getClassResolver();
-    resolver.registerSerializer(Date.class, new DateSerializer(fury));
-    resolver.registerSerializer(java.sql.Date.class, new SqlDateSerializer(fury));
-    resolver.registerSerializer(Time.class, new SqlTimeSerializer(fury));
-    resolver.registerSerializer(Timestamp.class, new TimestampSerializer(fury));
-    resolver.registerSerializer(LocalDate.class, new LocalDateSerializer(fury));
-    resolver.registerSerializer(LocalTime.class, new LocalTimeSerializer(fury));
-    resolver.registerSerializer(LocalDateTime.class, new LocalDateTimeSerializer(fury));
-    resolver.registerSerializer(Instant.class, new InstantSerializer(fury));
-    resolver.registerSerializer(Duration.class, new DurationSerializer(fury));
-    resolver.registerSerializer(ZoneOffset.class, new ZoneOffsetSerializer(fury));
-    resolver.registerSerializer(ZonedDateTime.class, new ZonedDateTimeSerializer(fury));
-    resolver.registerSerializer(Year.class, new YearSerializer(fury));
-    resolver.registerSerializer(YearMonth.class, new YearMonthSerializer(fury));
-    resolver.registerSerializer(MonthDay.class, new MonthDaySerializer(fury));
-    resolver.registerSerializer(Period.class, new PeriodSerializer(fury));
-    resolver.registerSerializer(OffsetTime.class, new OffsetTimeSerializer(fury));
-    resolver.registerSerializer(OffsetDateTime.class, new OffsetDateTimeSerializer(fury));
+    fury.registerSerializer(Date.class, new DateSerializer(fury));
+    fury.registerSerializer(java.sql.Date.class, new SqlDateSerializer(fury));
+    fury.registerSerializer(Time.class, new SqlTimeSerializer(fury));
+    fury.registerSerializer(Timestamp.class, new TimestampSerializer(fury));
+    fury.registerSerializer(LocalDate.class, new LocalDateSerializer(fury));
+    fury.registerSerializer(LocalTime.class, new LocalTimeSerializer(fury));
+    fury.registerSerializer(LocalDateTime.class, new LocalDateTimeSerializer(fury));
+    fury.registerSerializer(Instant.class, new InstantSerializer(fury));
+    fury.registerSerializer(Duration.class, new DurationSerializer(fury));
+    fury.registerSerializer(ZoneOffset.class, new ZoneOffsetSerializer(fury));
+    fury.registerSerializer(ZonedDateTime.class, new ZonedDateTimeSerializer(fury));
+    fury.registerSerializer(Year.class, new YearSerializer(fury));
+    fury.registerSerializer(YearMonth.class, new YearMonthSerializer(fury));
+    fury.registerSerializer(MonthDay.class, new MonthDaySerializer(fury));
+    fury.registerSerializer(Period.class, new PeriodSerializer(fury));
+    fury.registerSerializer(OffsetTime.class, new OffsetTimeSerializer(fury));
+    fury.registerSerializer(OffsetDateTime.class, new OffsetDateTimeSerializer(fury));
   }
 }
