@@ -46,8 +46,8 @@ import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.reflect.TypeRef;
 import org.apache.fury.resolver.ClassInfo;
 import org.apache.fury.resolver.ClassInfoHolder;
+import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.resolver.RefResolver;
-import org.apache.fury.resolver.TypeResolver;
 import org.apache.fury.serializer.Serializer;
 import org.apache.fury.type.GenericType;
 import org.apache.fury.type.Generics;
@@ -70,7 +70,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
   // or one of types is already instantiated in subclass, ex: `Subclass<T> implements Map<String,
   // T>`
   private final IdentityMap<GenericType, GenericType> partialGenericKVTypeMap;
-  private final GenericType objType;
+  private final GenericType objType = fury.getClassResolver().buildGenericType(Object.class);
   // For subclass whose kv type are instantiated already, such as
   // `Subclass implements Map<String, Long>`. If declared `Map` doesn't specify
   // instantiated kv type, then the serialization will need to write those kv
@@ -80,29 +80,33 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
   // interpreter and jit mode although it seems unnecessary.
   // With kv header in future, we can write this kv classes only once, the cost won't be too much.
   private int numElements;
-  private final TypeResolver typeResolver;
-  protected final SerializationBinding binding;
+  private final ClassResolver classResolver;
 
   public AbstractMapSerializer(Fury fury, Class<T> cls) {
     this(fury, cls, !ReflectionUtils.isDynamicGeneratedCLass(cls));
   }
 
   public AbstractMapSerializer(Fury fury, Class<T> cls, boolean supportCodegenHook) {
-    this(fury, cls, supportCodegenHook, false);
+    super(fury, cls);
+    this.classResolver = fury.getClassResolver();
+    this.supportCodegenHook = supportCodegenHook;
+    keyClassInfoWriteCache = fury.getClassResolver().nilClassInfoHolder();
+    keyClassInfoReadCache = fury.getClassResolver().nilClassInfoHolder();
+    valueClassInfoWriteCache = fury.getClassResolver().nilClassInfoHolder();
+    valueClassInfoReadCache = fury.getClassResolver().nilClassInfoHolder();
+    partialGenericKVTypeMap = new IdentityMap<>();
   }
 
   public AbstractMapSerializer(
       Fury fury, Class<T> cls, boolean supportCodegenHook, boolean immutable) {
     super(fury, cls, immutable);
-    this.typeResolver = fury.isCrossLanguage() ? fury.getXtypeResolver() : fury.getClassResolver();
+    this.classResolver = fury.getClassResolver();
     this.supportCodegenHook = supportCodegenHook;
-    keyClassInfoWriteCache = typeResolver.nilClassInfoHolder();
-    keyClassInfoReadCache = typeResolver.nilClassInfoHolder();
-    valueClassInfoWriteCache = typeResolver.nilClassInfoHolder();
-    valueClassInfoReadCache = typeResolver.nilClassInfoHolder();
+    keyClassInfoWriteCache = fury.getClassResolver().nilClassInfoHolder();
+    keyClassInfoReadCache = fury.getClassResolver().nilClassInfoHolder();
+    valueClassInfoWriteCache = fury.getClassResolver().nilClassInfoHolder();
+    valueClassInfoReadCache = fury.getClassResolver().nilClassInfoHolder();
     partialGenericKVTypeMap = new IdentityMap<>();
-    objType = typeResolver.buildGenericType(Object.class);
-    binding = SerializationBinding.createBinding(fury);
   }
 
   /**
@@ -134,7 +138,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
     if (map.isEmpty()) {
       return;
     }
-    TypeResolver classResolver = typeResolver;
+    ClassResolver classResolver = fury.getClassResolver();
     Iterator<Entry<Object, Object>> iterator = map.entrySet().iterator();
     Entry<Object, Object> entry = iterator.next();
     while (entry != null) {
@@ -161,7 +165,8 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
 
   @Override
   public void xwrite(MemoryBuffer buffer, T value) {
-    write(buffer, value);
+    Map map = onMapWrite(buffer, value);
+    xwriteElements(fury, buffer, map);
   }
 
   public final Entry writeJavaNullChunk(
@@ -194,14 +199,14 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
     if (keySerializer != null) {
       if (keySerializer.needToWriteRef()) {
         buffer.writeByte(NULL_VALUE_KEY_DECL_TYPE_TRACKING_REF);
-        binding.writeRef(buffer, key, keySerializer);
+        fury.writeRef(buffer, key, keySerializer);
       } else {
         buffer.writeByte(NULL_VALUE_KEY_DECL_TYPE);
-        binding.write(buffer, keySerializer, key);
+        keySerializer.write(buffer, key);
       }
     } else {
       buffer.writeByte(VALUE_HAS_NULL | TRACKING_KEY_REF);
-      binding.writeRef(buffer, key, keyClassInfoWriteCache);
+      fury.writeRef(buffer, key, keyClassInfoWriteCache);
     }
   }
 
@@ -216,14 +221,14 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
       if (valueSerializer != null) {
         if (valueSerializer.needToWriteRef()) {
           buffer.writeByte(NULL_KEY_VALUE_DECL_TYPE_TRACKING_REF);
-          binding.writeRef(buffer, value, valueSerializer);
+          fury.writeRef(buffer, value, valueSerializer);
         } else {
           buffer.writeByte(NULL_KEY_VALUE_DECL_TYPE);
-          binding.write(buffer, valueSerializer, value);
+          valueSerializer.write(buffer, value);
         }
       } else {
         buffer.writeByte(KEY_HAS_NULL | TRACKING_VALUE_REF);
-        binding.writeRef(buffer, value, valueClassInfoWriteCache);
+        fury.writeRef(buffer, value, valueClassInfoWriteCache);
       }
     } else {
       buffer.writeByte(KV_NULL);
@@ -245,7 +250,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
           return entry;
         }
         buffer.writeByte(NULL_VALUE_KEY_DECL_TYPE);
-        binding.write(buffer, keySerializer, key);
+        keySerializer.write(buffer, key);
       } else {
         writeNullKeyChunk(buffer, valueSerializer, value);
       }
@@ -259,7 +264,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
 
   // Make byte code of this method smaller than 325 for better jit inline
   private Entry writeJavaChunk(
-      TypeResolver classResolver,
+      ClassResolver classResolver,
       MemoryBuffer buffer,
       Entry<Object, Object> entry,
       Iterator<Entry<Object, Object>> iterator,
@@ -304,10 +309,10 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
         break;
       }
       if (!keyWriteRef || !refResolver.writeRefOrNull(buffer, key)) {
-        binding.write(buffer, keySerializer, key);
+        keySerializer.write(buffer, key);
       }
       if (!valueWriteRef || !refResolver.writeRefOrNull(buffer, value)) {
-        binding.write(buffer, valueSerializer, value);
+        valueSerializer.write(buffer, value);
       }
       // noinspection Duplicates
       ++chunkSize;
@@ -328,21 +333,21 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
   }
 
   private Serializer writeKeyClassInfo(
-      TypeResolver classResolver, Class keyType, MemoryBuffer buffer) {
+      ClassResolver classResolver, Class keyType, MemoryBuffer buffer) {
     ClassInfo classInfo = classResolver.getClassInfo(keyType, keyClassInfoWriteCache);
-    classResolver.writeClassInfo(buffer, classInfo);
+    classResolver.writeClass(buffer, classInfo);
     return classInfo.getSerializer();
   }
 
   private Serializer writeValueClassInfo(
-      TypeResolver classResolver, Class valueType, MemoryBuffer buffer) {
+      ClassResolver classResolver, Class valueType, MemoryBuffer buffer) {
     ClassInfo classInfo = classResolver.getClassInfo(valueType, valueClassInfoWriteCache);
-    classResolver.writeClassInfo(buffer, classInfo);
+    classResolver.writeClass(buffer, classInfo);
     return classInfo.getSerializer();
   }
 
   private Entry writeJavaChunkGeneric(
-      TypeResolver classResolver,
+      ClassResolver classResolver,
       Generics generics,
       GenericType genericType,
       MemoryBuffer buffer,
@@ -418,14 +423,14 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
       generics.pushGenericType(keyGenericType);
       if (!keyWriteRef || !refResolver.writeRefOrNull(buffer, key)) {
         fury.incDepth(1);
-        binding.write(buffer, keySerializer, key);
+        keySerializer.write(buffer, key);
         fury.incDepth(-1);
       }
       generics.popGenericType();
       generics.pushGenericType(valueGenericType);
       if (!valueWriteRef || !refResolver.writeRefOrNull(buffer, value)) {
         fury.incDepth(1);
-        binding.write(buffer, valueSerializer, value);
+        valueSerializer.write(buffer, value);
         fury.incDepth(-1);
       }
       generics.popGenericType();
@@ -447,6 +452,66 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
     return entry;
   }
 
+  public static void xwriteElements(Fury fury, MemoryBuffer buffer, Map value) {
+    Generics generics = fury.getGenerics();
+    GenericType genericType = generics.nextGenericType();
+    // TODO(chaokunyang) support map subclass whose key or value generics only are available.
+    if (genericType == null || genericType.getTypeParametersCount() != 2) {
+      for (Object object : value.entrySet()) {
+        Map.Entry entry = (Map.Entry) object;
+        fury.xwriteRef(buffer, entry.getKey());
+        fury.xwriteRef(buffer, entry.getValue());
+      }
+    } else {
+      // TODO(chaokunyang) use codegen to remove all branches.
+      GenericType keyGenericType = genericType.getTypeParameter0();
+      GenericType valueGenericType = genericType.getTypeParameter1();
+      Serializer keySerializer = keyGenericType.getSerializer(fury.getClassResolver());
+      Serializer valueSerializer = valueGenericType.getSerializer(fury.getClassResolver());
+      if (!keyGenericType.hasGenericParameters() && !valueGenericType.hasGenericParameters()) {
+        for (Object object : value.entrySet()) {
+          Map.Entry entry = (Map.Entry) object;
+          xwriteRefByNullableSerializer(fury, buffer, entry.getKey(), keySerializer);
+          xwriteRefByNullableSerializer(fury, buffer, entry.getValue(), valueSerializer);
+        }
+      } else if (valueGenericType.hasGenericParameters()) {
+        for (Object object : value.entrySet()) {
+          Map.Entry entry = (Map.Entry) object;
+          xwriteRefByNullableSerializer(fury, buffer, entry.getKey(), keySerializer);
+          generics.pushGenericType(valueGenericType);
+          xwriteRefByNullableSerializer(fury, buffer, entry.getValue(), valueSerializer);
+          generics.popGenericType();
+        }
+      } else if (keyGenericType.hasGenericParameters()) {
+        for (Object object : value.entrySet()) {
+          Map.Entry entry = (Map.Entry) object;
+          generics.pushGenericType(keyGenericType);
+          xwriteRefByNullableSerializer(fury, buffer, entry.getKey(), keySerializer);
+          generics.popGenericType();
+          xwriteRefByNullableSerializer(fury, buffer, entry.getValue(), valueSerializer);
+        }
+      } else {
+        for (Object object : value.entrySet()) {
+          Map.Entry entry = (Map.Entry) object;
+          generics.pushGenericType(keyGenericType);
+          xwriteRefByNullableSerializer(fury, buffer, entry.getKey(), keySerializer);
+          generics.pushGenericType(valueGenericType);
+          xwriteRefByNullableSerializer(fury, buffer, entry.getValue(), valueSerializer);
+        }
+      }
+      generics.popGenericType();
+    }
+  }
+
+  public static <T> void xwriteRefByNullableSerializer(
+      Fury fury, MemoryBuffer buffer, T obj, Serializer<T> serializer) {
+    if (serializer == null) {
+      fury.xwriteRef(buffer, obj);
+    } else {
+      fury.xwriteRef(buffer, obj, serializer);
+    }
+  }
+
   private GenericType getKVGenericType(GenericType genericType) {
     GenericType mapGenericType = partialGenericKVTypeMap.get(genericType);
     if (mapGenericType == null) {
@@ -465,11 +530,13 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
 
   @Override
   public T xread(MemoryBuffer buffer) {
-    return read(buffer);
+    Map map = newMap(buffer);
+    xreadElements(fury, buffer, map, numElements);
+    return onMapRead(map);
   }
 
   protected <K, V> void copyEntry(Map<K, V> originMap, Map<K, V> newMap) {
-    TypeResolver classResolver = typeResolver;
+    ClassResolver classResolver = fury.getClassResolver();
     for (Map.Entry<K, V> entry : originMap.entrySet()) {
       K key = entry.getKey();
       if (key != null) {
@@ -491,7 +558,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
   }
 
   protected <K, V> void copyEntry(Map<K, V> originMap, Builder<K, V> builder) {
-    TypeResolver classResolver = typeResolver;
+    ClassResolver classResolver = fury.getClassResolver();
     for (Entry<K, V> entry : originMap.entrySet()) {
       K key = entry.getKey();
       if (key != null) {
@@ -513,7 +580,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
   }
 
   protected <K, V> void copyEntry(Map<K, V> originMap, Object[] elements) {
-    TypeResolver classResolver = typeResolver;
+    ClassResolver classResolver = fury.getClassResolver();
     int index = 0;
     for (Entry<K, V> entry : originMap.entrySet()) {
       K key = entry.getKey();
@@ -600,12 +667,12 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
           Object key;
           if ((chunkHeader & KEY_DECL_TYPE) != 0) {
             if (trackKeyRef) {
-              key = binding.readRef(buffer, keySerializer);
+              key = fury.readRef(buffer, keySerializer);
             } else {
-              key = binding.read(buffer, keySerializer);
+              key = keySerializer.read(buffer);
             }
           } else {
-            key = binding.readRef(buffer, keyClassInfoReadCache);
+            key = fury.readRef(buffer, keyClassInfoReadCache);
           }
           map.put(key, null);
         }
@@ -636,12 +703,12 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
       boolean trackValueRef = (chunkHeader & TRACKING_VALUE_REF) != 0;
       if ((chunkHeader & VALUE_DECL_TYPE) != 0) {
         if (trackValueRef) {
-          value = binding.readRef(buffer, valueSerializer);
+          value = fury.readRef(buffer, valueSerializer);
         } else {
-          value = binding.read(buffer, valueSerializer);
+          value = valueSerializer.read(buffer);
         }
       } else {
-        value = binding.readRef(buffer, valueClassInfoReadCache);
+        value = fury.readRef(buffer, valueClassInfoReadCache);
       }
       map.put(null, value);
     } else {
@@ -664,7 +731,7 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
         if (!valueHasNull) {
           return (size << 8) | chunkHeader;
         } else {
-          Object key = binding.read(buffer, keySerializer);
+          Object key = keySerializer.read(buffer);
           map.put(key, null);
         }
       } else {
@@ -693,20 +760,19 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
     boolean valueIsDeclaredType = (chunkHeader & VALUE_DECL_TYPE) != 0;
     int chunkSize = buffer.readUnsignedByte();
     if (!keyIsDeclaredType) {
-      keySerializer = typeResolver.readClassInfo(buffer, keyClassInfoReadCache).getSerializer();
+      keySerializer = classResolver.readClassInfo(buffer, keyClassInfoReadCache).getSerializer();
     }
     if (!valueIsDeclaredType) {
-      valueSerializer = typeResolver.readClassInfo(buffer, valueClassInfoReadCache).getSerializer();
+      valueSerializer =
+          classResolver.readClassInfo(buffer, valueClassInfoReadCache).getSerializer();
     }
     for (int i = 0; i < chunkSize; i++) {
-      Object key =
-          trackKeyRef
-              ? binding.readRef(buffer, keySerializer)
-              : binding.read(buffer, keySerializer);
+      // increase depth to avoid read wrong outer generic type
+      fury.incDepth(1);
+      Object key = trackKeyRef ? fury.readRef(buffer, keySerializer) : keySerializer.read(buffer);
       Object value =
-          trackValueRef
-              ? binding.readRef(buffer, valueSerializer)
-              : binding.read(buffer, valueSerializer);
+          trackValueRef ? fury.readRef(buffer, valueSerializer) : valueSerializer.read(buffer);
+      fury.incDepth(-1);
       map.put(key, value);
       size--;
     }
@@ -737,56 +803,95 @@ public abstract class AbstractMapSerializer<T> extends Serializer<T> {
     int chunkSize = buffer.readUnsignedByte();
     Serializer keySerializer, valueSerializer;
     if (!keyIsDeclaredType) {
-      keySerializer = typeResolver.readClassInfo(buffer, keyClassInfoReadCache).getSerializer();
+      keySerializer = classResolver.readClassInfo(buffer, keyClassInfoReadCache).getSerializer();
     } else {
-      keySerializer = keyGenericType.getSerializer(typeResolver);
+      keySerializer = keyGenericType.getSerializer(classResolver);
     }
     if (!valueIsDeclaredType) {
-      valueSerializer = typeResolver.readClassInfo(buffer, valueClassInfoReadCache).getSerializer();
+      valueSerializer =
+          classResolver.readClassInfo(buffer, valueClassInfoReadCache).getSerializer();
     } else {
-      valueSerializer = valueGenericType.getSerializer(typeResolver);
+      valueSerializer = valueGenericType.getSerializer(classResolver);
     }
-    if (keyGenericType.hasGenericParameters() || valueGenericType.hasGenericParameters()) {
-      for (int i = 0; i < chunkSize; i++) {
-        generics.pushGenericType(keyGenericType);
-        fury.incDepth(1);
-        Object key =
-            trackKeyRef
-                ? binding.readRef(buffer, keySerializer)
-                : binding.read(buffer, keySerializer);
-        fury.incDepth(-1);
-        generics.popGenericType();
-        generics.pushGenericType(valueGenericType);
-        fury.incDepth(1);
-        Object value =
-            trackValueRef
-                ? binding.readRef(buffer, valueSerializer)
-                : binding.read(buffer, valueSerializer);
-        fury.incDepth(-1);
-        generics.popGenericType();
-        map.put(key, value);
-        size--;
-      }
-    } else {
-      for (int i = 0; i < chunkSize; i++) {
-        // increase depth to avoid read wrong outer generic type
-        fury.incDepth(1);
-        Object key =
-            trackKeyRef
-                ? binding.readRef(buffer, keySerializer)
-                : binding.read(buffer, keySerializer);
-        Object value =
-            trackValueRef
-                ? binding.readRef(buffer, valueSerializer)
-                : binding.read(buffer, valueSerializer);
-        fury.incDepth(-1);
-        map.put(key, value);
-        size--;
-      }
+    for (int i = 0; i < chunkSize; i++) {
+      generics.pushGenericType(keyGenericType);
+      fury.incDepth(1);
+      Object key = trackKeyRef ? fury.readRef(buffer, keySerializer) : keySerializer.read(buffer);
+      fury.incDepth(-1);
+      generics.popGenericType();
+      generics.pushGenericType(valueGenericType);
+      fury.incDepth(1);
+      Object value =
+          trackValueRef ? fury.readRef(buffer, valueSerializer) : valueSerializer.read(buffer);
+      fury.incDepth(-1);
+      generics.popGenericType();
+      map.put(key, value);
+      size--;
     }
     return size > 0 ? (size << 8) | buffer.readUnsignedByte() : 0;
   }
 
+  @SuppressWarnings("unchecked")
+  public static void xreadElements(Fury fury, MemoryBuffer buffer, Map map, int size) {
+    Generics generics = fury.getGenerics();
+    GenericType genericType = generics.nextGenericType();
+    if (genericType == null || genericType.getTypeParametersCount() != 2) {
+      for (int i = 0; i < size; i++) {
+        Object key = fury.xreadRef(buffer);
+        Object value = fury.xreadRef(buffer);
+        map.put(key, value);
+      }
+    } else {
+      // TODO(chaokunyang) use codegen to remove all branches.
+      GenericType keyGenericType = genericType.getTypeParameter0();
+      GenericType valueGenericType = genericType.getTypeParameter1();
+      Serializer keySerializer = keyGenericType.getSerializer(fury.getClassResolver());
+      Serializer valueSerializer = valueGenericType.getSerializer(fury.getClassResolver());
+      if (!keyGenericType.hasGenericParameters() && !valueGenericType.hasGenericParameters()) {
+        for (int i = 0; i < size; i++) {
+          Object key = xreadRefByNullableSerializer(fury, buffer, keySerializer);
+          Object value = xreadRefByNullableSerializer(fury, buffer, valueSerializer);
+          map.put(key, value);
+        }
+      } else if (valueGenericType.hasGenericParameters()) {
+        for (int i = 0; i < size; i++) {
+          Object key = xreadRefByNullableSerializer(fury, buffer, keySerializer);
+          generics.pushGenericType(valueGenericType);
+          Object value = xreadRefByNullableSerializer(fury, buffer, valueSerializer);
+          generics.popGenericType();
+          map.put(key, value);
+        }
+      } else if (keyGenericType.hasGenericParameters()) {
+        for (int i = 0; i < size; i++) {
+          generics.pushGenericType(keyGenericType);
+          Object key = xreadRefByNullableSerializer(fury, buffer, keySerializer);
+          generics.popGenericType();
+          Object value = xreadRefByNullableSerializer(fury, buffer, valueSerializer);
+          map.put(key, value);
+        }
+      } else {
+        for (int i = 0; i < size; i++) {
+          // FIXME(chaokunyang) nested generics may be get by mistake.
+          generics.pushGenericType(keyGenericType);
+          Object key = xreadRefByNullableSerializer(fury, buffer, keySerializer);
+          generics.pushGenericType(valueGenericType);
+          Object value = xreadRefByNullableSerializer(fury, buffer, valueSerializer);
+          map.put(key, value);
+        }
+      }
+      generics.popGenericType();
+    }
+  }
+
+  public static Object xreadRefByNullableSerializer(
+      Fury fury, MemoryBuffer buffer, Serializer<?> serializer) {
+    if (serializer == null) {
+      return fury.xreadRef(buffer);
+    } else {
+      return fury.xreadRef(buffer, serializer);
+    }
+  }
+
   /**
    * Hook for java serialization codegen, read/write key/value by entrySet.
    *
