@@ -28,7 +28,6 @@ import org.apache.fury.logging.Logger;
 import org.apache.fury.logging.LoggerFactory;
 import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.reflect.ReflectionUtils;
-import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.serializer.ObjectSerializer;
 
 @SuppressWarnings({"rawtypes", "unchecked"})
@@ -65,15 +64,14 @@ public class SubListSerializers {
   }
 
   public static void registerSerializers(Fury fury, boolean preserveView) {
-    ClassResolver classResolver = fury.getClassResolver();
     // java.util.ImmutableCollections$SubList is already registered in
     // ImmutableCollectionSerializers
     for (Class<?> cls :
         new Class[] {SubListClass, RandomAccessSubListClass, ArrayListSubListClass}) {
       if (fury.trackingRef() && preserveView && fury.getConfig().getLanguage() == Language.JAVA) {
-        classResolver.registerSerializer(cls, new SubListViewSerializer(fury, cls));
+        fury.registerSerializer(cls, new SubListViewSerializer(fury, cls));
       } else {
-        classResolver.registerSerializer(cls, new SubListSerializer(fury, (Class<List>) cls));
+        fury.registerSerializer(cls, new SubListSerializer(fury, (Class<List>) cls));
       }
     }
   }
