@@ -40,7 +40,6 @@ import org.apache.fury.logging.Logger;
 import org.apache.fury.logging.LoggerFactory;
 import org.apache.fury.memory.MemoryBuffer;
 import org.apache.fury.memory.Platform;
-import org.apache.fury.resolver.ClassResolver;
 import org.apache.fury.serializer.Serializer;
 import org.apache.fury.util.ExceptionUtils;
 
@@ -212,9 +211,8 @@ public class SynchronizedSerializers {
    */
   public static void registerSerializers(Fury fury) {
     try {
-      ClassResolver resolver = fury.getClassResolver();
       for (Tuple2<Class<?>, Function> factory : synchronizedFactories()) {
-        resolver.registerSerializer(factory.f0, createSerializer(fury, factory));
+        fury.registerSerializer(factory.f0, createSerializer(fury, factory));
       }
     } catch (Throwable e) {
       ExceptionUtils.ignore(e);
