@@ -78,7 +78,6 @@ public class Descriptor {
   private Class<?> type;
   private final String typeName;
   private final String name;
-  private String snakeCaseName;
   private final int modifier;
   private final String declaringClass;
   private final Field field;
@@ -155,13 +154,6 @@ public class Descriptor {
     return name;
   }
 
-  public String getSnakeCaseName() {
-    if (snakeCaseName == null) {
-      snakeCaseName = StringUtils.lowerCamelToLowerUnderscore(name);
-    }
-    return snakeCaseName;
-  }
-
   public int getModifiers() {
     return modifier;
   }
@@ -214,7 +206,7 @@ public class Descriptor {
     sb.append(", name=").append(name);
     sb.append(", modifier=").append(modifier);
     if (field != null) {
-      sb.append(", declaringClass=").append(field.getDeclaringClass().getSimpleName());
+      sb.append(", field=").append(field.getDeclaringClass().getSimpleName()).append('.');
     }
     if (readMethod != null) {
       sb.append(", readMethod=").append(readMethod);
