@@ -41,17 +41,15 @@ import org.apache.fury.util.record.RecordUtils;
  * <li>other fields
  */
 public class DescriptorGrouper {
-  static final Comparator<Descriptor> COMPARATOR_BY_PRIMITIVE_TYPE_ID =
+  // sort primitive descriptors from largest to smallest, if size is the same,
+  // sort by field name to fix order.
+  private static final Comparator<Descriptor> PRIMITIVE_COMPARATOR =
       (d1, d2) -> {
         int c =
-            Types.getPrimitiveTypeId(TypeUtils.unwrap(d2.getRawType()))
-                - Types.getPrimitiveTypeId(TypeUtils.unwrap(d1.getRawType()));
+            getSizeOfPrimitiveType(TypeUtils.unwrap(d2.getRawType()))
+                - getSizeOfPrimitiveType(TypeUtils.unwrap(d1.getRawType()));
         if (c == 0) {
-          c = d1.getSnakeCaseName().compareTo(d2.getSnakeCaseName());
-          if (c == 0) {
-            // Field name duplicate in super/child classes.
-            c = d1.getDeclaringClass().compareTo(d2.getDeclaringClass());
-          }
+          c = DescriptorGrouper.COMPARATOR_BY_TYPE_AND_NAME.compare(d1, d2);
         }
         return c;
       };
@@ -66,17 +64,7 @@ public class DescriptorGrouper {
   public static Comparator<Descriptor> getPrimitiveComparator(
       boolean compressInt, boolean compressLong) {
     if (!compressInt && !compressLong) {
-      // sort primitive descriptors from largest to smallest, if size is the same,
-      // sort by field name to fix order.
-      return (d1, d2) -> {
-        int c =
-            getSizeOfPrimitiveType(TypeUtils.unwrap(d2.getRawType()))
-                - getSizeOfPrimitiveType(TypeUtils.unwrap(d1.getRawType()));
-        if (c == 0) {
-          c = COMPARATOR_BY_PRIMITIVE_TYPE_ID.compare(d1, d2);
-        }
-        return c;
-      };
+      return PRIMITIVE_COMPARATOR;
     }
     return (d1, d2) -> {
       Class<?> t1 = TypeUtils.unwrap(d1.getRawType());
@@ -86,7 +74,7 @@ public class DescriptorGrouper {
       if ((t1Compress && t2Compress) || (!t1Compress && !t2Compress)) {
         int c = getSizeOfPrimitiveType(t2) - getSizeOfPrimitiveType(t1);
         if (c == 0) {
-          c = COMPARATOR_BY_PRIMITIVE_TYPE_ID.compare(d1, d2);
+          c = DescriptorGrouper.COMPARATOR_BY_TYPE_AND_NAME.compare(d1, d2);
         }
         return c;
       }
@@ -143,18 +131,16 @@ public class DescriptorGrouper {
   /**
    * Create a descriptor grouper.
    *
-   * @param isMonomorphic whether the class is monomorphic.
    * @param descriptors descriptors may have field with same name.
-   * @param descriptorsGroupedOrdered whether the descriptors are grouped and ordered.
-   * @param descriptorUpdater create a new descriptor from original one.
+   * @param descriptorUpdator create a new descriptor from original one.
    * @param primitiveComparator comparator for primitive/boxed fields.
    * @param comparator comparator for non-primitive fields.
    */
-  private DescriptorGrouper(
+  public DescriptorGrouper(
       Predicate<Class<?>> isMonomorphic,
       Collection<Descriptor> descriptors,
       boolean descriptorsGroupedOrdered,
-      Function<Descriptor, Descriptor> descriptorUpdater,
+      Function<Descriptor, Descriptor> descriptorUpdator,
       Comparator<Descriptor> primitiveComparator,
       Comparator<Descriptor> comparator) {
     this.primitiveDescriptors =
@@ -169,18 +155,18 @@ public class DescriptorGrouper {
     this.otherDescriptors =
         descriptorsGroupedOrdered ? new ArrayList<>() : new TreeSet<>(comparator);
     for (Descriptor descriptor : descriptors) {
-      if (TypeUtils.isPrimitive(descriptor.getRawType())) {
-        primitiveDescriptors.add(descriptorUpdater.apply(descriptor));
+      if (descriptor.getRawType().isPrimitive()) {
+        primitiveDescriptors.add(descriptorUpdator.apply(descriptor));
       } else if (TypeUtils.isBoxed(descriptor.getRawType())) {
-        boxedDescriptors.add(descriptorUpdater.apply(descriptor));
+        boxedDescriptors.add(descriptorUpdator.apply(descriptor));
       } else if (TypeUtils.isCollection(descriptor.getRawType())) {
-        collectionDescriptors.add(descriptorUpdater.apply(descriptor));
+        collectionDescriptors.add(descriptorUpdator.apply(descriptor));
       } else if (TypeUtils.isMap(descriptor.getRawType())) {
-        mapDescriptors.add(descriptorUpdater.apply(descriptor));
+        mapDescriptors.add(descriptorUpdator.apply(descriptor));
       } else if (isMonomorphic.test(descriptor.getRawType())) {
-        finalDescriptors.add(descriptorUpdater.apply(descriptor));
+        finalDescriptors.add(descriptorUpdator.apply(descriptor));
       } else {
-        otherDescriptors.add(descriptorUpdater.apply(descriptor));
+        otherDescriptors.add(descriptorUpdator.apply(descriptor));
       }
     }
   }
@@ -190,9 +176,9 @@ public class DescriptorGrouper {
     descriptors.addAll(getPrimitiveDescriptors());
     descriptors.addAll(getBoxedDescriptors());
     descriptors.addAll(getFinalDescriptors());
-    descriptors.addAll(getOtherDescriptors());
     descriptors.addAll(getCollectionDescriptors());
     descriptors.addAll(getMapDescriptors());
+    descriptors.addAll(getOtherDescriptors());
     return descriptors;
   }
 
@@ -236,17 +222,16 @@ public class DescriptorGrouper {
       Predicate<Class<?>> isMonomorphic,
       Collection<Descriptor> descriptors,
       boolean descriptorsGroupedOrdered,
-      Function<Descriptor, Descriptor> descriptorUpdator,
       boolean compressInt,
-      boolean compressLong,
-      Comparator<Descriptor> comparator) {
+      boolean compressLong) {
+    Comparator<Descriptor> comparator = getPrimitiveComparator(compressInt, compressLong);
     return new DescriptorGrouper(
         isMonomorphic,
         descriptors,
         descriptorsGroupedOrdered,
-        descriptorUpdator == null ? DescriptorGrouper::createDescriptor : descriptorUpdator,
-        getPrimitiveComparator(compressInt, compressLong),
-        comparator);
+        DescriptorGrouper::createDescriptor,
+        comparator,
+        COMPARATOR_BY_TYPE_AND_NAME);
   }
 
   public int getNumDescriptors() {
