@@ -30,7 +30,6 @@ import org.apache.fury.reflect.TypeRef;
 public class ScalaTypes {
   private static volatile Class<?> SCALA_MAP_TYPE;
   private static volatile Class<?> SCALA_SEQ_TYPE;
-  private static volatile Class<?> SCALA_SET_TYPE;
   private static volatile Class<?> SCALA_ITERABLE_TYPE;
   private static volatile java.lang.reflect.Type SCALA_ITERATOR_RETURN_TYPE;
   private static volatile java.lang.reflect.Type SCALA_NEXT_RETURN_TYPE;
@@ -51,13 +50,6 @@ public class ScalaTypes {
     return SCALA_SEQ_TYPE;
   }
 
-  public static Class<?> getScalaSetType() {
-    if (SCALA_SET_TYPE == null) {
-      SCALA_SET_TYPE = ReflectionUtils.loadClass("scala.collection.Set");
-    }
-    return SCALA_SET_TYPE;
-  }
-
   public static Class<?> getScalaIterableType() {
     if (SCALA_ITERABLE_TYPE == null) {
       SCALA_ITERABLE_TYPE = ReflectionUtils.loadClass("scala.collection.Iterable");
