@@ -50,9 +50,7 @@ import org.apache.fury.collection.Tuple2;
 import org.apache.fury.reflect.ReflectionUtils;
 import org.apache.fury.reflect.TypeParameter;
 import org.apache.fury.reflect.TypeRef;
-import org.apache.fury.serializer.NonexistentClass;
 import org.apache.fury.util.Preconditions;
-import org.apache.fury.util.StringUtils;
 
 /** Type utils for common type inference and extraction. */
 @SuppressWarnings({"UnstableApiUsage", "unchecked"})
@@ -554,9 +552,6 @@ public class TypeUtils {
   }
 
   public static boolean isMap(Class<?> cls) {
-    if (cls == NonexistentClass.NonexistentMetaShared.class) {
-      return false;
-    }
     return cls == HashMap.class || Map.class.isAssignableFrom(cls);
   }
 
@@ -768,12 +763,4 @@ public class TypeUtils {
     }
     return getArrayComponent(clz).isEnum();
   }
-
-  public static String qualifiedName(String pkg, String className) {
-    if (StringUtils.isBlank(pkg)) {
-      return className;
-    } else {
-      return pkg + "." + className;
-    }
-  }
 }
