@@ -19,7 +19,6 @@
 
 package org.apache.fury.serializer;
 
-import static org.apache.fury.reflect.ReflectionUtils.getObjectFieldValue;
 import static org.apache.fury.serializer.ClassUtils.loadClass;
 
 import com.google.common.collect.ImmutableSet;
@@ -55,7 +54,7 @@ import org.testng.annotations.Test;
  * interoperability between them.
  */
 public class MetaSharedCompatibleTest extends FuryTestBase {
-  public static Object serDeMetaSharedCheck(Fury fury, Object obj) {
+  public static Object serDeCheck(Fury fury, Object obj) {
     Object newObj = serDeMetaShared(fury, obj);
     Assert.assertEquals(newObj, obj);
     return newObj;
@@ -116,9 +115,9 @@ public class MetaSharedCompatibleTest extends FuryTestBase {
             .withRefTracking(referenceTracking)
             .withCodegen(enableCodegen)
             .build();
-    serDeMetaSharedCheck(fury, Foo.create());
-    serDeMetaSharedCheck(fury, BeanB.createBeanB(2));
-    serDeMetaSharedCheck(fury, BeanA.createBeanA(2));
+    serDeCheck(fury, Foo.create());
+    serDeCheck(fury, BeanB.createBeanB(2));
+    serDeCheck(fury, BeanA.createBeanA(2));
   }
 
   @Test(dataProvider = "config2")
@@ -674,210 +673,65 @@ public class MetaSharedCompatibleTest extends FuryTestBase {
         new ClassDefEncoderTest
             .TestClassLengthTestClassLengthTestClassLengthTestClassLengthTestClassLengthTestClassLengthTestClassLength
             .InnerClassTestLengthInnerClassTestLengthInnerClassTestLength();
-    serDeMetaSharedCheck(fury, o);
-  }
-
-  CompileUnit aunit =
-      new CompileUnit(
-          "demo.pkg1",
-          "A",
-          (""
-              + "package demo.pkg1;\n"
-              + "import demo.test.*;\n"
-              + "import demo.report.*;\n"
-              + "public class A {\n"
-              + "  int f1;\n"
-              + "  B f2;\n"
-              + "  C f3;\n"
-              + "  public String toString() {return \"A\" + \",\" + f1 + \",\" + f2 + \",\" + f3;}\n"
-              + "  public static A create() { A a = new A(); a.f1 = 10; a.f2 = new B(); a.f3 = new C(); return a;}\n"
-              + "}"));
-
-  CompileUnit bunit =
-      new CompileUnit(
-          "demo.test",
-          "B",
-          ("" + "package demo.test;\n" + "public class B {\n" + "  public int f1 = 100;\n" + "}"));
-
-  CompileUnit cunit =
-      new CompileUnit(
-          "demo.report",
-          "C",
-          (""
-              + "package demo.report;\n"
-              + "public class C {\n"
-              + "  public int f2 = 1000;\n"
-              + "  public String toString() {return \"C{f2=\" + f2 + \"}\";}\n"
-              + "}"));
-
-  CompileUnit newAUnit =
-      new CompileUnit(
-          "example.pkg1",
-          "A",
-          (""
-              + "package example.pkg1;\n"
-              + "import example.test.*;\n"
-              + "import example.report.*;\n"
-              + "public class A {\n"
-              + "  public int f1;\n"
-              + "  public C f3;\n"
-              + "  public String toString() {return \"A\" + \",\" + f1 + \",\" + f3;}\n"
-              + "  public static A create() { A a = new A(); a.f1 = 10; a.f3 = new C(); return a;}\n"
-              + "}"));
-
-  CompileUnit newCUnit =
-      new CompileUnit(
-          "example.test",
-          "C",
-          (""
-              + "package example.test;\n"
-              + "public class C {\n"
-              + "  public int f2;\n"
-              + "  public String toString() {return \"C{f2=\" + f2 + \"}\";}\n"
-              + "}"));
-
-  @Test
-  public void testRegisterToSameIdForRenamedClass() throws Exception {
-    ClassLoader classLoader =
-        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), aunit, bunit, cunit);
-    byte[] serialized;
-    {
-      Class<?> A = classLoader.loadClass("demo.pkg1.A");
-      Class<?> B = classLoader.loadClass("demo.test.B");
-      Class<?> C = classLoader.loadClass("demo.report.C");
-      Fury fury =
-          builder()
-              .withCompatibleMode(CompatibleMode.COMPATIBLE)
-              .withClassLoader(classLoader)
-              .build();
-      fury.register(A);
-      fury.register(B);
-      fury.register(C);
-      Object a = A.getMethod("create").invoke(null);
-      System.out.println(a);
-      serialized = fury.serialize(a);
-    }
-    {
-      CompileUnit unit1 =
-          new CompileUnit(
-              "example.pkg1",
-              "A",
-              (""
-                  + "package example.pkg1;\n"
-                  + "import example.test.*;\n"
-                  + "import example.report.*;\n"
-                  + "public class A {\n"
-                  + "  int f1;\n"
-                  + "  B f2;\n"
-                  + "  C f3;\n"
-                  + "  public String toString() {return \"A\" + \",\" + f1 + \",\" + f2 + \",\" + f3;}\n"
-                  + "  public static A create() { A a = new A(); a.f1 = 10; a.f2 = new B(); a.f3 = new C(); return a;}\n"
-                  + "}"));
-      CompileUnit unit2 =
-          new CompileUnit(
-              "example.report",
-              "B",
-              ("" + "package example.report;\n" + "public class B {\n" + "}"));
-      CompileUnit unit3 =
-          new CompileUnit(
-              "example.test", "C", ("" + "package example.test;\n" + "public class C {\n" + "}"));
-      classLoader =
-          JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), unit1, unit2, unit3);
-      Class<?> A = classLoader.loadClass("example.pkg1.A");
-      Class<?> B = classLoader.loadClass("example.report.B");
-      Class<?> C = classLoader.loadClass("example.test.C");
-      Fury fury =
-          builder()
-              .withCompatibleMode(CompatibleMode.COMPATIBLE)
-              .withClassLoader(classLoader)
-              .build();
-      fury.register(A);
-      fury.register(B);
-      fury.register(C);
-      Object newObj = fury.deserialize(serialized);
-      System.out.println(newObj);
-    }
+    serDeCheck(fury, o);
   }
 
   @Test
-  public void testInconsistentRegistrationID() throws Exception {
-    ClassLoader classLoader =
-        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), aunit, bunit, cunit);
-    byte[] serialized;
-    {
-      Class<?> A = classLoader.loadClass("demo.pkg1.A");
-      Class<?> B = classLoader.loadClass("demo.test.B");
-      Class<?> C = classLoader.loadClass("demo.report.C");
-      Fury fury =
-          builder()
-              .withCompatibleMode(CompatibleMode.COMPATIBLE)
-              .withClassLoader(classLoader)
-              .build();
-      fury.register(A, 300);
-      fury.register(B, 301);
-      fury.register(C, 302);
-      Object a = A.getMethod("create").invoke(null);
-      System.out.println(a);
-      serialized = fury.serialize(a);
-    }
-    {
-      classLoader =
-          JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), newAUnit, newCUnit);
-      Class<?> A = classLoader.loadClass("example.pkg1.A");
-      Class<?> C = classLoader.loadClass("example.test.C");
-      Fury fury =
-          builder()
-              .withCompatibleMode(CompatibleMode.COMPATIBLE)
-              .withClassLoader(classLoader)
-              .build();
-      fury.register(A, 300);
-      fury.register(C, 302);
-      Object newObj = fury.deserialize(serialized);
-      System.out.println(newObj);
-      Object f3 = getObjectFieldValue(newObj, "f3");
-      Assert.assertNotNull(f3);
-      Assert.assertEquals(f3.toString(), "C{f2=1000}");
-    }
-  }
-
-  @Test
-  public void testInconsistentRegistrationName() throws Exception {
-    ClassLoader classLoader =
-        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), aunit, bunit, cunit);
-    byte[] serialized;
-    {
-      Class<?> A = classLoader.loadClass("demo.pkg1.A");
-      Class<?> B = classLoader.loadClass("demo.test.B");
-      Class<?> C = classLoader.loadClass("demo.report.C");
-      Fury fury =
-          builder()
-              .withCompatibleMode(CompatibleMode.COMPATIBLE)
-              .withClassLoader(classLoader)
-              .build();
-      fury.register(A, "test.A");
-      fury.register(B, "test.B");
-      fury.register(C, "test.C");
-      Object a = A.getMethod("create").invoke(null);
-      System.out.println(a);
-      serialized = fury.serialize(a);
-    }
-    {
-      classLoader =
-          JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), newAUnit, newCUnit);
-      Class<?> A = classLoader.loadClass("example.pkg1.A");
-      Class<?> C = classLoader.loadClass("example.test.C");
-      Fury fury =
-          builder()
-              .withCompatibleMode(CompatibleMode.COMPATIBLE)
-              .withClassLoader(classLoader)
-              .build();
-      fury.register(A, "test.A");
-      fury.register(C, "test.C");
-      Object newObj = fury.deserialize(serialized);
-      System.out.println(newObj);
-      Object f3 = getObjectFieldValue(newObj, "f3");
-      Assert.assertNotNull(f3);
-      Assert.assertEquals(f3.toString(), "C{f2=1000}");
-    }
+  public void testInheritance() throws Exception {
+    // See issue https://github.com/apache/fury/issues/2210
+    CompileUnit u1 =
+        new CompileUnit(
+            "example.test",
+            "Parent",
+            (""
+                + "package example.test;\n"
+                + "public class Parent {\n"
+                + "  public Integer f1;\n"
+                + "  public Integer f2;\n"
+                + "}"));
+    CompileUnit u2 =
+        new CompileUnit(
+            "example.test",
+            "Child",
+            (""
+                + "package example.test;\n"
+                + "public class Child extends Parent {\n"
+                + "  public Integer f3;\n"
+                + "}"));
+    ClassLoader classLoader1 =
+        JaninoUtils.compile(Thread.currentThread().getContextClassLoader(), u1, u2);
+    Object o = classLoader1.loadClass("example.test.Child").newInstance();
+    ReflectionUtils.setObjectFieldValue(o, "f1", 10);
+    ReflectionUtils.setObjectFieldValue(o, "f2", 20);
+    ReflectionUtils.setObjectFieldValue(o, "f3", 30);
+    Fury fury =
+        builder()
+            .withCompatibleMode(CompatibleMode.COMPATIBLE)
+            .withClassLoader(classLoader1)
+            .build();
+    fury.register(classLoader1.loadClass("example.test.Child"));
+    fury.register(classLoader1.loadClass("example.test.Parent"));
+    byte[] serialized = fury.serialize(o);
+    CompileUnit u11 =
+        new CompileUnit(
+            "example.test",
+            "Parent",
+            (""
+                + "package example.test;\n"
+                + "public class Parent {\n"
+                + "  public Integer f2;\n"
+                + "}"));
+    ClassLoader classLoader2 = JaninoUtils.compile(getClass().getClassLoader(), u11, u2);
+    Fury fury2 =
+        builder()
+            .withCompatibleMode(CompatibleMode.COMPATIBLE)
+            .withClassLoader(classLoader2)
+            .build();
+    fury2.register(classLoader2.loadClass("example.test.Child"));
+    fury2.register(classLoader2.loadClass("example.test.Parent"));
+    Object o1 = fury2.deserialize(serialized);
+    Assert.assertNull(ReflectionUtils.getObjectFieldValue(o1, "f1"));
+    Assert.assertEquals(ReflectionUtils.getObjectFieldValue(o1, "f2"), 20);
+    Assert.assertEquals(ReflectionUtils.getObjectFieldValue(o1, "f3"), 30);
   }
 }
