@@ -57,14 +57,17 @@ public class SerializersTest extends FuryTestBase {
     Fury fury1 = builder.build();
     Fury fury2 = builder.build();
     assertEquals("str", serDe(fury1, fury2, "str"));
-    assertEquals("str", serDeObject(fury1, fury2, new StringBuilder("str")).toString());
-    assertEquals("str", serDeObject(fury1, fury2, new StringBuffer("str")).toString());
+    assertEquals("str", serDe(fury1, fury2, new StringBuilder("str")).toString());
+    assertEquals("str", serDe(fury1, fury2, new StringBuffer("str")).toString());
   }
 
-  @Test(dataProvider = "referenceTrackingConfig")
-  public void testBigInt(boolean referenceTracking) {
+  @Test(dataProvider = "crossLanguageReferenceTrackingConfig")
+  public void testBigInt(boolean referenceTracking, Language language) {
     FuryBuilder builder =
-        Fury.builder().withRefTracking(referenceTracking).requireClassRegistration(false);
+        Fury.builder()
+            .withLanguage(language)
+            .withRefTracking(referenceTracking)
+            .requireClassRegistration(false);
     Fury fury1 = builder.build();
     Fury fury2 = builder.build();
     assertEquals(BigInteger.valueOf(100), serDe(fury1, fury2, BigInteger.valueOf(100)));
